# Copyright (C) 2009 Chris Larson <clarson@kergoth.com>
# Released under the MIT license (see COPYING.MIT for the terms)
#
# gitver.bbclass provides a GITVER variable which is a (fairly) sane version,
# for use in ${PV}, extracted from the ${S} git checkout, assuming it is one.

# Assume that the Git repository is in ${S} unless the recipe tells us otherwise
GITVER_S ?= "${S}"

TAG_MACHINE = "${MACHINE}"
# If we're building the tests, get the tag from the underlying machine instead
TAG_MACHINE_cheetah-tests = "cheetah"
TAG_MACHINE_panther-tests = "panther"
TAG_MACHINE_qemux86 = "panther"

BS_GITVER = "${@get_git_pv_notags('${GITVER_S}', d)}"

def get_git_pv_notags(path, d):
    import os
    import bb.process
    import time
    import re
    import calendar

    rootdir = os.path.abspath(path)
    try:
        need_machine = d.getVar('COMPATIBLE_MACHINE', True)
        if need_machine:
            import re
            compat_machines = (d.getVar('MACHINEOVERRIDES', True) or "").split(":")
            for m in compat_machines:
                if re.match(need_machine, m):
                    break
            else:
                return "0.0"

        # This checks to see if there are any files in the working
        # tree or the index that are different to HEAD. If they are
        # then we tack +local on the end of the version to indicate
        # that there are local changes. This stops us matching
        # anything from the shared sstate cache. We don't make PV
        # change based on the mtime of the most recently changed file
        # because that would cause us to fetch the source again every
        # time which can often be avoided during development.
        git_env = os.environ.copy()
        git_env["PSEUDO_DISABLED"] = "1"
        git_env["PSEUDO_UNLOAD"] = "1"

        # Avoid bug #17631 during incremental/nightly builds
        never_modified = d.getVar('BS_GITVER_NEVER_MODIFIED', True)
        if never_modified and never_modified == "1":
            maybe_dirty = ""
        else:
            args = [ "(/usr/bin/git diff --quiet && /usr/bin/git diff --cached --quiet) || echo '+local'" ]
            maybe_dirty = bb.process.run(args, cwd=rootdir, env=git_env, shell=True)[0].rstrip()
        
        args = [ "/usr/bin/git", "show", "-s", "--pretty=%ci", "HEAD" ]
        ver = bb.process.run(args, cwd=rootdir, env=git_env)[0].rstrip()

        args = [ "/usr/bin/git", "rev-parse", "--short", "HEAD" ]
        hash = bb.process.run(args, cwd=rootdir, env=git_env)[0].rstrip()

        r = re.compile("(.*) ([\+-])([0-9][0-9])([0-9][0-9])")
        m = r.match(ver)
        ts = time.strptime(m.group(1), "%Y-%m-%d %H:%M:%S")
        dstoff = int(m.group(4)) * 60 + int(m.group(3)) * 3600
        if (m.group(2) == '-'):
            dstoff = -dstoff
        tv = int(calendar.timegm(ts))
        tv -= dstoff
        ts = time.gmtime(tv)
        ver = time.strftime("%Y%m%dT%H%M%S", ts) + "+" + hash + maybe_dirty
    except Exception, exc:
        # Don't fail here during recipe parsing
        bb.warn("%s(%s): %s" % (d.getVar("PN", True), rootdir, exc))
        ver = "0.0-not-in-git"

    return ver

__BB_DONT_CACHE = "1"
