# Calculate a hash of the headers installed into ${includedir} to
# detect changes that may constitute ABI changes and fail the build.
#
# Recipes using this class must define HEADERS_SIGNATURE with the
# expected signature.

do_headers_signature() {
    set -x
    cd ${D}

    # This generates a sorted list of files which are hashed
    # individually by sha1sum to generate a "manifest". This
    # "manifest" is then itself hashed to yield a single hash that is
    # used for comparison.
    calculated="`find ./${includedir} -type f -print0|sort -z|xargs -0 sha1sum|sha1sum -|cut -c-40`"
    if [ "$calculated" != "${HEADERS_SIGNATURE}" ]; then
        echo "Header signature mismatch" 1>&2
        echo "  Calculated: $calculated" 1>&2
        echo "  Expected:   ${HEADERS_SIGNATURE}" 1>&2
        exit 1
    fi
}

addtask do_headers_signature after do_install before do_package

# People won't notice that they've changed the signatures if they use
# purely nfsquick unless we add the task here

NFSQUICK_FUNCS += "do_headers_signature"
